// This is a small open economy model that corresponds to a simplified version of
// the Christiano-Trabandt-Walentin model.
// It produces impulse response graphs to various shocks 

// Endogenous variables in handout (the equations for x, PHI are used, not x and PHI themselves).
var lpm lpc lq lpx lR lc lpibar ls af logN 
lpstar lF lK ly lpic 

// Other endogenous variables included for graphing purposes.
lexp limp lc_homog

// Exogenous variables 
da lRf  lyf lg ltaud lpitarget
lpif phitilde;

// Shock innovations
varexo da_eps lRf_eps
lyf_eps lg_eps ltaud_eps lpitarget_eps lpif_eps phitilde_eps
money_eps;

parameters beta muz theta taud epsilon vrphi omegac etac etag etaa
 phi phitildea phitildes etaf rhoR rpi ry v pitarget
 lpm_SS lpc_SS lq_SS lpx_SS lR_SS lc_SS lpibar_SS ls_SS af_SS logN_SS 
 lpstar_SS lF_SS lK_SS ly_SS lpic_SS lexp_SS limp_SS
 da_SS lRf_SS lyf_SS lg_SS ltaud_SS lpitarget_SS
 lpif_SS phitilde_SS ;

// loads up parameters and steady states
% Do not put ; at the end of the line
@# include "open_economy_parameters.mod"

lexp_SS=-etaf*lpx_SS+lyf_SS+lpc_SS+lq_SS+lpx_SS;

limp_SS=lpm_SS+log(omegac)+etac*(lpc_SS-lpm_SS)+lc_SS;

model;
// first number is equation number according to Dynare
// eq# is the number of the equation in the handout, Christiano, 'Small Open Economy'

//1 eq1 
exp(-lpstar)=(1-theta)*((1-theta*exp(lpibar*(epsilon-1)))/(1-theta))^(epsilon/(epsilon-1))+theta*exp(epsilon*lpibar-lpstar(-1));

//2 eq2
exp(lF)-exp(ly-lc)-beta*theta*exp(lpibar(+1)*(epsilon-1)+lF(+1)); 

//3 eq3
exp(lK-lF)=((1-theta*exp(lpibar*(epsilon-1)))/(1-theta))^(1/(1-epsilon)); 

//4 eq4 
exp(lK)-(epsilon/(epsilon-1))*(1-v)*exp(ltaud+phi*logN+ly) 
-beta*theta*exp(lpibar(+1)*epsilon+lK(+1)); 

//5 eq5
lR-lR_SS=rhoR*(lR(-1)-lR_SS)+(1-rhoR)*(rpi*(lpic(+1)-lpitarget_SS)+
                 ry*(ly(+1)-ly_SS))+money_eps;

//6 eq6 & eq16 
exp(-lc)=beta/(exp(da(+1)+lpic(+1)+lc(+1)))
*(exp(ls(+1))*exp(lRf)* 
exp(-phitildea*(af-af_SS)-phitildes*(exp(lRf)-exp(lR)-(exp(lRf_SS)-exp(lR_SS)))+phitilde));

//7 eq7 
exp(-lc)-beta*exp(lR-lc(+1)-lpic(+1)-da(+1));

//8 eq8 
exp(lpc)-((1-omegac)+omegac*exp(lpm*(1-etac)))^(1/(1-etac));

//9 eq9  
lpm=lpc+lq;

//10 eq10
exp(lpic)=exp(lpibar)* 
((1-omegac+omegac*(exp(lpm))^(1-etac))/ 
(1-omegac+omegac*(exp(lpm(-1)))^(1-etac)))^(1/(1-etac)); 

//11 eq11 & eq14  
-exp(ly) + exp(lg) + (1-omegac)*exp(lpc*etac+lc) + exp(-etaf*lpx+lyf);

//12 eq12  
0=lq+lpc+lpx;

//13 eq13  
exp(lq)/exp(lq(-1))=exp(ls)*exp(lpif)/exp(lpic);

//14 eq15 & eq16
af+exp(lpm)*omegac*exp(etac*(lpc-lpm)+lc)
  = exp(lq+lpc+(1-etaf)*lpx+lyf) + exp(lRf(-1)-phitildea*(af(-1)-af_SS) 
  -phitildes*(exp(lRf(-1))-exp(lR(-1))-(exp(lRf_SS)-exp(lR_SS))) 
  +phitilde(-1))*exp(ls-lpibar-da)*af(-1);  

//15 exports, in units of domestic homogeneous goods. Value lpc+lq+lpx=ln(S*PX/P) is export price
lexp=-etaf*lpx+lyf+lpc+lq+lpx;

//16 imports, in units of the domestic homogeneous good
limp=lpm+log(omegac)+etac*(lpc-lpm)+lc;

// 17 homogeneous output
ly = lpstar + logN;

// 18 log consumption, measured in homogeneous goods units

lc_homog=lc+lpc;

// 19 - 26
da = (1-.95)*da_SS + .95*da(-1) + da_eps;
lRf = (1-.95)*lRf_SS + .95*lRf(-1) + lRf_eps;
lyf = (1-.95)*lyf_SS + .95*lyf(-1) + lyf_eps;
lpif = (1-.95)*lpif_SS + .95*lpif(-1) + lpif_eps;
lg = (1-.95)*(log(etag)+ly_SS) + .95*lg(-1) + lg_eps;
//lg = (1-1.85+0.855)*(log(etag)+ly_SS) + 1.85*lg(-1)-0.855*lg(-2)+lg_eps;
ltaud = (1-.95)*ltaud_SS + .95*ltaud(-1) + ltaud_eps;
lpitarget = (1-.95)*lpitarget_SS + .95*lpitarget(-1) + lpitarget_eps;
phitilde = .95*phitilde(-1) + phitilde_eps;
end;

shocks;
var da_eps; 
stderr 0.01;
var lyf_eps;
stderr 0.01;
var lpitarget_eps;
stderr 0.01;
var lpif_eps;
stderr 0.01;
var money_eps;
stderr 0.01;
var ltaud_eps;
stderr 0.01;
var phitilde_eps;
stderr 0.01;
var lRf_eps;
stderr 0.01;
var lg_eps;
stderr 0.01;
end;


initval;
lpm=lpm_SS;
lpc=lpc_SS;
lq=lq_SS;
lpx=lpx_SS;
lR=lR_SS;
lc=lc_SS;
lpibar=lpibar_SS;
ls=ls_SS;
af=af_SS;
logN=logN_SS;
lpstar=lpstar_SS;
lF=lF_SS;
lK=lK_SS;
ly=ly_SS;
lpic=lpic_SS;
lexp=lexp_SS;
limp=limp_SS;
da=da_SS;
lRf=lRf_SS;
lyf=lyf_SS;
lg=lg_SS;
ltaud=ltaud_SS;
lpif=lpif_SS;
phitilde = phitilde_SS;
lpitarget=lpitarget_SS;
lc_homog=lc_SS+lpc_SS;
end;

steady;
set_dynare_seed=1;
stoch_simul(irf=20,order=1,nograph) ly lc_homog lR logN lpic ls lq af lpx lexp limp lg lRf lpibar;

//the following code can be rearranged to show impulse responses for any of the 12 shocks defined in the shocks command. 

figure('name','foreign interest rate')
pltout(ly_lRf_eps,lc_homog_lRf_eps,lR_lRf_eps,lR_SS,ls_lRf_eps,logN_lRf_eps,lq_lRf_eps,af_lRf_eps,lpx_lRf_eps,lpic_lRf_eps,lpic_SS,lexp_lRf_eps,limp_lRf_eps);
hout=suptitle('response to foreign interest rate shock');

figure('name','government consumption')
pltout(ly_lg_eps,lc_homog_lg_eps,lR_lg_eps,lR_SS,ls_lg_eps,logN_lg_eps,lq_lg_eps,af_lg_eps,lpx_lg_eps,lpic_lg_eps,lpic_SS,lexp_lg_eps,limp_lg_eps);
hout=suptitle('response to government consumption shock');

figure('name','country risk premium')
pltout(ly_phitilde_eps,lc_homog_phitilde_eps,lR_phitilde_eps,lR_SS,ls_phitilde_eps,logN_phitilde_eps,lq_phitilde_eps,af_phitilde_eps,lpx_phitilde_eps,lpic_phitilde_eps,lpic_SS,lexp_phitilde_eps,limp_phitilde_eps);                                                             
hout=suptitle('response to country risk premium shock');

figure('name','monetary policy')
pltout(ly_money_eps,lc_homog_money_eps,lR_money_eps,lR_SS,ls_money_eps,logN_money_eps,lq_money_eps,af_money_eps,lpx_money_eps,lpic_money_eps,lpic_SS,lexp_money_eps,limp_money_eps);                                                                                   
hout=suptitle('response to monetary policy shock');

figure('name','foreign demand')
pltout(ly_lyf_eps,lc_homog_lyf_eps,lR_lyf_eps,lR_SS,ls_lyf_eps,logN_lyf_eps,lq_lyf_eps,af_lyf_eps,lpx_lyf_eps,lpic_lyf_eps,lpic_SS,lexp_lyf_eps,limp_lyf_eps);                                                                                   
hout=suptitle('jump in foreign demand');

